/***************************************************************************
                          messagedispatcher.h  -  description
                             -------------------
    begin                : Fri Jan 19 2001
    copyright            : (C) 2001 by Matthias Welwarsky
    email                : matze@stud.fbi.fh-darmstadt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef MESSAGEDISPATCHER_H
#define MESSAGEDISPATCHER_H

#include <qobject.h>
#include <qlist.h>
#include <qregexp.h>

class PhoneConnection;
class PhoneControl;

/** the message dispatcher diverts messages from the mobile phone to the phone controls that have registered for a special type of message.
 *  Registration for messages is done by supplying a QRegex object matching the appropriate message prefix
 *  @author Matthias Welwarsky
 */
class MessageDispatcher : public QObject {
	Q_OBJECT

public:
	MessageDispatcher(PhoneConnection*);
	~MessageDispatcher();

    /** Acquires the dispatcher for a phone control. This must be done before sending messages, for locking
      * purposes. Arguments are a QObject* and a slot definition. All responses from the conversation with
      * the phone are emitted to the QObjects slot, until releaseDispatcher is called.
      */
    bool acquireDispatcher(QObject*, const char*);

    /** releases the dispatcher. The Phone Control can no longer send any messages to the phone unless it
      * acquires the dispatcher again.
      */
    bool releaseDispatcher(QObject*);

signals:
	void responseReady(QString);

public slots: // Public slots
  	/** called by the PhoneConnection when a complete response line is received from the mobile phone. */
  	void receiveResponse(QString);
  	/** called by the PhoneControl object to send a command to the mobile phone */
  	void sendCommand(QString, QObject*);
  	/** transmit a PDU to the mobile phone. the only difference to sendCommand is the line termination,
	  * which is "CR" for normal commands, and "ESC" for PDUs */
  	void sendPDU(QString, QObject*);

private: // Private attributes
  	/** holds a pointer to the associated phone connection. */
  	PhoneConnection* phoneConnection;

    /** holds a pointer to the control that has currently acquired the dispatcher. NULL if no control has
      * acquired the dispatcher. */
    QObject* currentControl;
};

#endif

